# Using simcf and tile to explore an estimated logistic regression
# (download libraries from chrisadolph.com)
# Chris Adolph
# chrisadolph.com
#
# May 2014
     
################################################################
#
# Prob of Afr Slavery, minerals and low indig hierarchy
# 

# Clear memory
rm(list=ls())

# Load libraries
library(MASS)
library(simcf)
library(tile)
library(RColorBrewer)

library(lpSolve)
library(WhatIf)

# In Stata: use "indig_origins_data.dta" 
#           outsheet using "indig_origins_data.csv", comma nolabel replace
# Rename as data2014.csv

# Load data
file <- "data2014.csv"
data <- read.csv(file, header=TRUE)

# Estimate logit model using glm()
# Set up model formula and model specific data frame
model <- ps_slv_afr ~  il1 + il2 + minerals + lpopdens2 + sspain + sport + suk + sfra + sneth + logalt + abslat + logdist + expedition_arrival + governor_arrival + il1*minerals +il2*minerals 
mdata <- extractdata(model, data, na.rm=TRUE)

# Run logit & extract results
logit.result <- glm(model, family=binomial, data=mdata)
pe.glm <- logit.result$coefficients  # point estimates
vc.glm <- vcov(logit.result)         # var-cov matrix

# Import var-cov matrix
vc <- "vc.afrslv4.csv"  # var-cov matrix for Conley standard errors
vc.afrslv <- read.csv(vc, header=TRUE,row.names=1)   # var-cov matrix


## Simulate quantities of interest using simcf
##
## We could do this from the optim or glm results;
## here, we do from glm

# Simulate parameter distributions
sims <- 10000
simbetas <- mvrnorm(sims, pe.glm, vc.afrslv)

# Set up counterfactuals:
xhyp <- seq(0,1,0.2)
nscen <- length(xhyp)
LhierScen <- MhierScen2 <- HhierScen <- cfMake(model, mdata, nscen)
for (i in 1:nscen) {
  # L hierarchy scenarios
  LhierScen <- cfChange(LhierScen, "il2", x = 0, scen = i)
  LhierScen <- cfChange(LhierScen, "il1", x = 1, scen = i)
  LhierScen <- cfChange(LhierScen, "minerals", x = xhyp[i], scen = i)  

}

# Simulate expected probabilities 
LhierSims <- logitsimev(LhierScen, simbetas, ci=0.90)

# Get 3 nice colors for traces
col <- brewer.pal(3,"Greys") #brewer.pal(3,"Dark2") 

# Set up lineplot traces of expected probabilities

LhierTrace <- lineplot(x=xhyp,
                      y=LhierSims$pe,
                      lower=LhierSims$lower,
                      upper=LhierSims$upper,
                      col=col[3],
                      ci=list(mark="dashed"),
                      plot=1)


# Set up traces with labels and legend
labelTrace <- textTile(labels=c("A"),
                      x=c(0.95),
                       y=c(0.95),
                       plot=1)


# Plot traces using tile
tile(LhierTrace,
     labelTrace,
     limits=c(0,1,0,1),
     xaxis=list(at=c(0,1),labels=c("No","Yes")),
     yaxis=list(label.loc=-0.5, major=FALSE),
     xaxistitle=list(labels="Minerals"),
     yaxistitle=list(labels="Probability of African Slavery"),
     #maintitle=list(labels="Low Pre-Colonial Hierarchy",x=0.6),
     width=list(null=5,yaxistitle=4,yaxis.labelspace=-0.5),
     output=list(file="L_5",width=5.5,family="Times")
     )

     
################################################################
#
# Prob of Afr Slavery, minerals and high indig hierarchy
# 

# Simulate parameter distributions
sims <- 10000
simbetas <- mvrnorm(sims, pe.glm, vc.afrslv)

# Set up counterfactuals: 
xhyp <- seq(0,1,0.2)
nscen <- length(xhyp)
LhierScen <- MhierScen2 <- HhierScen <- cfMake(model, mdata, nscen)
for (i in 1:nscen) {
   
  # H hierarchy scenarios
  HhierScen <- cfChange(HhierScen, "il2", x = 0, scen = i)
  HhierScen <- cfChange(HhierScen, "il1", x = 0, scen = i)
  HhierScen <- cfChange(HhierScen, "minerals", x = xhyp[i], scen = i)  
}


# Simulate expected probabilities 
HhierSims <- logitsimev(HhierScen, simbetas, ci=0.90)

# Get 3 nice colors for traces
col <- brewer.pal(3,"Greys") #brewer.pal(3,"Dark2") 

# Set up lineplot traces of expected probabilities

HhierTrace <- lineplot(x=xhyp,
                      y=HhierSims$pe,
                      lower=HhierSims$lower,
                      upper=HhierSims$upper,
                      col=col[3],
                      ci=list(mark="dashed"),
                      plot=1)


# Set up traces with labels and legend
labelHTrace <- textTile(labels=c("B"),
                      x=c(0.9),
                       y=c(0.95),
                       plot=2)


# Plot traces using tile
tile(HhierTrace,
     labelTrace,
     limits=c(0,1,0,1),
     xaxis=list(at=c(0,1),labels=c("No","Yes")),
     yaxis=list(label.loc=-0.5, major=FALSE),
     xaxistitle=list(labels="Minerals"),
     yaxistitle=list(labels="Probability of African Slavery"),
     maintitle=list(labels="High Pre-Colonial Hierarchy",x=0.6),
     width=list(null=5,yaxistitle=4,yaxis.labelspace=-0.5),
     output=list(file="H_5",width=5.5,family="Times")
     )
     
     
################################################################
#
#  Side-by-side plots of L and H
#
labelTrace1 <- textTile(labels=c("A"),
                       x=c(0.95),
                       y=c(0.95),
                       plot=1)
                       
labelTrace2 <- textTile(labels=c("B"),
                       x=c(0.95),
                       y=c(0.95),
                       plot=2)

 
# Reassign RR traces to plot 2
HhierTrace$plot <- labelTrace2$plot <- 2


# Plot all traces using tile
# NOTE changes to limits, yaxistitles, and output width
tile(LhierTrace,
	 labelTrace1,
     HhierTrace,
      labelTrace2,
     limits=rbind(c(0,1,0,1),c(0,1,0,1)),
     frame=TRUE,
     xaxis=list(at=c(0,1),labels=c("No","Yes")),
     yaxis=list(label.loc=-0.5, major=FALSE),
     xaxistitle=list(labels="Minerals"),
     yaxistitle=list(labels1="Probability of African Slavery",
     				 labels2=""),
     plottitle=list(labels1="Low Pre-Colonial Hierarchy",labels2="High Pre-Colonial Hierarchy"),
     width=list(null=5,yaxistitle=4,yaxis.labelspace=-0.5),
     output=list(file="LH-5",width=10,family="Times")
     )
